from goby_sdk.V1.lib.base_page import BasePage


class ConfigPage(BasePage):

    def get_list_adapters(self):
        """
        获取可用的网卡列表

        :return: 返回网卡列表的JSON对象
        :rtype: dict
        """
        return self._send_get_request("api/v1/listAdapters")

    def get_environment_info(self):
        """
        获取系统环境信息

        :param field: 需要获取的字段，多个字段用逗号分隔，默认获取全部
        :type field: str
        :return: 返回系统环境信息的JSON对象
        :rtype: dict
        """
        # 查询参数
        field = 'chromePath,dataIntegrity,datadir,dir,midKey,proxyServer,userRuleSize,vulVersion,licenseInfo,gid,key,godserver'
        params = {
            'field': field
        }
        # 发送GET请求
        return self._send_get_request('api/v1/getEnvi', params=params)

    def set_environment_info(self, dns_server, proxy_server):
        """
        设置系统环境信息
        :param dns_server: DNS 服务器地址和端口，例如 "8.8.8.8:53"
        :type dns_server: str
        :param proxy_server: 代理服务器地址和端口，例如 "http://192.168.61.41:1080"
        :type proxy_server: str
        :return: 返回设置结果的 JSON 对象
        :rtype: dict
        """
        # 组装请求参数
        data = {
            "dnsServer": dns_server,
            "proxyServer": proxy_server
        }
        # 发送请求并返回响应结果
        return self._send_request("api/v1/setEnvi", data)



from goby_sdk.V1.lib.base_page import BasePage

class AssetPage(BasePage):
    def get_statistics_data(self, task_id):
        """
        获取任务的统计数据
        :param task_id: 任务ID
        :type task_id: str
        :return: 返回统计数据的JSON对象
        :rtype: dict
        """
        data = {
            "taskId": task_id
        }
        return self._send_request("api/v1/getStatisticsData", data)

    def asset_search(self, task_id, query='', page=1, page_size=20,
                     order_by={'vulnerabilities': 'desc', 'assets': 'desc'}):
        """
        根据查询条件搜索资产
        :param task_id: 任务ID
        :type task_id: str
        :param query: 查询条件，字符串形式，默认为空字符串
        :type query: str
        :param page: 分页页码，默认为1
        :type page: int
        :param page_size: 分页大小，默认为20
        :type page_size: int
        :param order_by: 排序字段和排序方式，默认为 {'vulnerabilities': 'desc', 'assets': 'desc'}
        :type order_by: dict
        :return: 返回搜索结果的JSON对象
        :rtype: dict
        """
        data = {
            "query": f"taskId=\"{task_id}\" && ({query})",
            "options": {
                "order": order_by,
                "page": {
                    "page": page,
                    "size": page_size
                }
            }
        }
        options = {'order': {'vulnerabilities': 'desc', 'assets': 'desc'}, 'page': {'page': 1, 'size': 20}}
        options = {**options, **order_by, 'page': {'page': page, 'size': page_size}}
        data['options'] = options

        return self._send_request("api/v1/assetSearch", data)

    def get_value_category(self, task_id):
        """
           获取任务值的分类
           :param task_id: 任务ID
           :type task_id: str
           :return: 返回任务列表的 JSON 对象
           :rtype: dict
           """
        data = {
            "taskId": task_id
        }
        return self._send_request("api/v1/getValueCategory", data)

    def get_children_category(self, task_id, parent_category):
        """
        获取子类别列表
        :param task_id: 任务ID
        :type task_id: str
        :param parent_category: 父类别
        :type parent_category: str
        :return: 返回JSON对象
        :rtype: dict
        """
        data = {
            "query": f"taskId=\"{task_id}\" && parent_category=\"{parent_category}\""
        }
        return self._send_request("api/v1/getChildrenCategory", data)

    def get_ip_info(self, task_id, ip):
        """
        获取IP信息
        :param task_id: 任务ID
        :type task_id: str
        :param ip: IP地址
        :type ip: str
        :return: 返回JSON对象
        :rtype: dict
        """
        data = {
            "taskId": task_id,
            "ip": ip
        }
        return self._send_request("api/v1/getIPInfo", data)

    def get_web_list(self, task_id, page=1, size=20):
        """
         获取任务的Web漏洞列表

         :param task_id: 任务ID
         :type task_id: str
         :param page: 页码，默认为1
         :type page: int
         :param size: 每页显示的记录数，默认为20
         :type size: int
         :return: 返回Web漏洞列表的JSON对象
         :rtype: dict
         """
        # 组装请求数据
        data = {
            "taskId": task_id,
            "options": {
                "page": {"page": page, "size": size}
            }
        }
        # 发送请求并返回响应结果
        return self._send_request("api/v1/getWebList", data)



from goby_sdk.V1.lib.base_page import BasePage


class VulnManagementPage(BasePage):

    def get_pocs(self, task_id, query, reload_pocs=False, order_by='vul_nums', order='desc', page=1, page_size=20):
        """
        获取POC列表

        :param task_id: 任务ID
        :type task_id: str
        :param query: 查询条件
        :type query: str
        :param reload_pocs: 是否重新加载POC，默认为False
        :type reload_pocs: bool
        :param order_by: 排序字段，默认为"vul_nums"
        :type order_by: str
        :param order: 排序方式，默认为"desc"
        :type order: str
        :param page: 页码，默认为1
        :type page: int
        :param page_size: 每页数量，默认为20
        :type page_size: int
        :return: 返回POC列表的JSON对象
        :rtype: dict
        """
        options = {
            "reloadPocs": reload_pocs,
            "order": {
                "vul_nums": "desc",
                "level": "desc",
                "host_nums": "desc"
            },
            "page": {
                "page": 1,
                "size": 20
            }
        }

        if order_by:
            options['order'][order_by] = order

        if page or page_size:
            options['page']['page'] = page
            options['page']['size'] = page_size

        data = {
            "taskId": task_id,
            "query": query,
            "options": options
        }

        return self._send_request("api/v1/getPocs", data)

    def get_failed_pocs(self):
        """
        获取失败的POC列表

        :return: 返回失败的POC列表的JSON对象
        :rtype: dict
        """
        return self._send_get_request("api/v1/getFailedPocs")

    def get_vuln_statistics(self, task_id):
        """
        获取任务的漏洞统计数据

        :param task_id: 任务ID
        :type task_id: str
        :return: 返回漏洞统计数据的JSON对象
        :rtype: dict
        """
        data = {"taskId": task_id}
        return self._send_request("api/v1/vulnerabilityStatisticsData", data)

    def get_vulnerability_search(self, task_id, page=1, size=20):
        """
        获取任务的漏洞搜索结果

        :param task_id: 任务ID
        :type task_id: str
        :param page: 搜索结果的页码，默认为1
        :type page: int
        :param size: 每页的搜索结果数量，默认为20
        :type size: int
        :return: 返回漏洞搜索结果的JSON对象
        :rtype: dict
        """
        data = {
            "taskId": task_id,
            "type": "vulnerability",
            "query": f"taskId=\"{task_id}\"",
            "options": {
                "order": {"level": "desc", "nums": "desc"},
                "page": {"page": page, "size": size}
            }
        }
        return self._send_request("api/v1/vulnerabilitySearch", data)

    def get_poc_info(self, vul_name):
        # 视乎还有个hostinfo参数，未添加
        """
        获取POC信息

        :param vul_name: 漏洞名称
        :type vul_name: str
        :return: 返回POC信息的JSON对象
        :rtype: dict
        """
        data = {"vulname": vul_name}
        return self._send_request("api/v1/getPOCInfo", data)



from goby_sdk.V1.lib.asset_page import AssetPage
from goby_sdk.V1.lib.home_page import HomePage
from goby_sdk.V1.lib.vulnerability_management_page import VulnManagementPage
from goby_sdk.V1.lib.report_page import ReportPage
from goby_sdk.V1.lib.config_page import ConfigPage


class Api:
    def __init__(self, url, username, password):
        self.HomePage = HomePage(url, username, password)
        self.AssetPage = AssetPage(url, username, password)
        self.VuleManagePage = VulnManagementPage(url, username, password)
        self.ReportPage = ReportPage(url, username, password)
        self.ConfigPage = ConfigPage(url, username, password)

    # 检测poc列表
    def get_poc_list(self, query="vultype=2", reload_pocs=False,
                     order_by={"vul_nums": "desc", "level": "desc", "host_nums": "desc"}, page=1, size=3000):
        return self.HomePage.get_poc_list(query=query, reload_pocs=reload_pocs, order_by=order_by, page=page, size=size)

    # 启动扫描
    def start_scan(self, task_name, ips, ports="0-65535", black_ips=[], vul_type="0", pocs_hosts={}, options={}):
        return self.HomePage.start_scan(task_name=task_name, ports=ports, ips=ips, black_ips=black_ips,
                                        vul_type=vul_type, pocs_hosts=pocs_hosts, options=options)

    # 重启扫描
    def resume_scan(self, task_id, options={}):
        return self.HomePage.resume_scan(task_id=task_id, options=options)

    # 停止扫描
    def stop_scan(self, task_id):
        return self.HomePage.stop_scan(task_id=task_id)

    # 查询扫描进度
    def get_scan_progress(self, task_id):
        return self.HomePage.get_scan_progress(task_id=task_id)

    # 获取任务列表
    def get_tasks(self, page=1, size=10, order_by='created_time', order='desc'):
        return self.HomePage.get_tasks(page=page, size=size, order_by=order_by, order=order)

    # 查询任务统计数据
    def get_statistics_data(self, task_id):
        return self.AssetPage.get_statistics_data(task_id=task_id)

    # 资产搜索
    def asset_search(self, task_id, query='', page=1, page_size=20,
                     order_by={'vulnerabilities': 'desc', 'assets': 'desc'}):
        return self.AssetPage.asset_search(task_id=task_id, query=query, page=page, page_size=page_size,
                                           order_by=order_by)

    # 获取产品类别
    def get_value_category(self, task_id):
        return self.AssetPage.get_value_category(task_id=task_id)

    # 获取支撑系统
    def get_children_category(self, task_id, parent_category):
        return self.AssetPage.get_children_category(task_id=task_id, parent_category=parent_category)

    # 获取IP详情
    def get_ip_info(self, task_id, ip):
        return self.AssetPage.get_ip_info(task_id=task_id, ip=ip)

    # 获取任务的Web漏洞列表
    def get_web_list(self, task_id, page=1, size=20):
        return self.AssetPage.get_web_list(task_id=task_id,page=page,size=size)

    # 获取漏洞信息
    def get_pocs(self, task_id, query, reload_pocs=False, order_by='vul_nums', order='desc', page=1, page_size=20):
        return self.VuleManagePage.get_pocs(task_id=task_id, query=query, reload_pocs=reload_pocs, order_by=order_by,
                                            order=order, page=page, page_size=page_size)

    # 获取失败的POC列表
    def get_failed_pocs(self):
        return self.VuleManagePage.get_failed_pocs()

    # 获取任务的漏洞统计数据
    def get_vuln_statistics(self,task_id):
        return self.VuleManagePage.get_vuln_statistics(task_id=task_id)

    # 获取任务的漏洞搜索结果
    def get_vulnerability_search(self,task_id, page=1, size=20):
        return self.VuleManagePage.get_vulnerability_search(task_id=task_id,page=page,size=size)

    # 获取POC信息
    def get_poc_info(self,vul_name):
        return self.VuleManagePage.get_poc_info(vul_name=vul_name)

    # 获取指定任务下的IP段信息
    def get_ip_segment(self, task_id, type):
        return self.ReportPage.get_ip_segment(task_id=task_id,segment_type=type)

    # 获取任务的漏洞分析数据
    def get_vul_analysis(self, task_id):
        return self.ReportPage.get_vul_analysis(task_id=task_id)

    # 获取指定任务的资产标签数据
    def get_asset_tags(self,task_id):
        return self.ReportPage.get_asset_tags(task_id=task_id)

    # 配置类：获取可用的网卡列表
    def get_list_adapters(self):
        return self.ConfigPage.get_list_adapters()

    # 配置类：获取系统环境信息
    def get_env_info(self):
        return self.ConfigPage.get_environment_info()

    #设置系统环境信息
    def set_env_info(self, dns_server, proxy_server):
        return self.ConfigPage.set_environment_info(dns_server=dns_server,proxy_server=proxy_server)


from goby_sdk.V1.lib.base_page import BasePage


class ReportPage(BasePage):

    def get_ip_segment(self, task_id, segment_type):
        """
        获取指定任务下的IP段信息

        :param task_id: 任务ID
        :type task_id: str
        :param segment_type: IP段类型，0表示所有，1表示已扫描，2表示未扫描
        :type segment_type: str
        :return: 返回IP段信息的JSON对象
        :rtype: dict
        """
        # 组装请求数据
        data = {
            "taskId": task_id,
            "type": segment_type
        }
        # 发送请求并返回响应结果
        return self._send_request("api/v1/ipSegment", data)

    def get_vul_analysis(self, task_id):
        """
        获取任务的漏洞分析数据

        :param task_id: 任务ID
        :type task_id: str
        :return: 返回漏洞分析数据的JSON对象
        :rtype: dict
        """
        # 组装请求数据
        data = {
            "taskId": task_id
        }
        # 发送请求并返回响应结果
        return self._send_request("api/v1/getVulAnalysis", data)

    def get_asset_tags(self, task_id):
        """
        获取指定任务的资产标签数据

        :param task_id: 任务ID
        :type task_id: str
        :return: 返回资产标签数据的JSON对象
        :rtype: dict
        """
        data = {"taskId": task_id}
        return self._send_request("api/v1/assetTags", data)



import requests
import base64
import json

class BasePage:
    def __init__(self, url, username, password):
        self.url = url
        self.username = username
        self.password = password
        self.session = requests.Session()

    def _send_request(self, path, data):
        authorization = base64.b64encode(f"{self.username}:{self.password}".encode()).decode()
        headers = {
            "Authorization": f"Basic {authorization}",
            "Content-Type": "application/json;charset=UTF-8",
        }
        response = self.session.post(f"{self.url}/{path}", headers=headers, data=json.dumps(data))
        return response.json()

    def _send_get_request(self, path, params=None):
        authorization = base64.b64encode(f"{self.username}:{self.password}".encode()).decode()
        headers = {
            "Authorization": f"Basic {authorization}",
            "Content-Type": "application/json;charset=UTF-8",
        }
        response = self.session.get(f"{self.url}/{path}", headers=headers, params=params)
        return response.json()


from setuptools import setup,find_packages

setup(
    name='goby_sdk',
    version='1.0.1',
    description='A Python SDK for Goby API',
    author='siberia',
    author_email='siberiah0h@gmail.com',
    packages=find_packages(),
    install_requires=[
        'requests>=2.25.1',
    ],
)


from goby_sdk.V1.lib.base_page import BasePage


class HomePage(BasePage):

    def get_poc_list(self, query="vultype=2", reload_pocs=False,
                     order_by={"vul_nums": "desc", "level": "desc", "host_nums": "desc"}, page=1, size=3000):
        """
        获取POC列表信息
        :param query: 查询条件，默认为"vultype=2"
        :type query: str
        :param reload_pocs: 是否重新加载POC，默认为False
        :type reload_pocs: bool
        :param order_by: 排序字段，默认为{"vul_nums": "desc", "level": "desc", "host_nums": "desc"}，可选值为"vul_nums"、"level"和"host_nums"
        :type order_by: dict
        :param page: 当前页码，默认为1
        :type page: int
        :param size: 每页显示的记录数，默认为3000
        :type size: int
        :return: 返回POC列表信息的JSON对象
        :rtype: dict
        """
        data = {
            "query": query,
            "options": {
                "reloadPocs": reload_pocs,
                "order": order_by,
                "page": {
                    "page": page,
                    "size": size
                }
            }
        }
        return self._send_request("api/v1/getPOCList", data)

    def start_scan(self, task_name, ips, ports="0-65535", black_ips=[], vul_type="0", pocs_hosts={}, options={}):
        """
        开始扫描任务
        :param task_name: 扫描任务名称
        :type task_name: str
        :param ips: 目标IP列表，多个IP以逗号分隔
        :type ips: str
        :param ports: 扫描端口范围，默认为"0-65535"
        :type ports: str
        :param black_ips: 黑名单IP列表，多个IP以逗号分隔，默认为空列表
        :type black_ips: list
        :param vul_type: 漏洞类型，默认为"0"，可选值为"0"（全部）、"1"（高危）、"2"（中危）、"3"（低危）
        :type vul_type: str
        :param pocs_hosts: POC插件列表，为空字典则使用系统内置插件，可通过get_poc_list方法获取插件列表
        :type pocs_hosts: dict
        :param options: 扫描选项，为字典类型，可配置的选项请参考API文档
        :type options: dict
        :return: 返回扫描任务信息的JSON对象
        :rtype: dict
        """
        asset = {"ips": ips.split(","), "ports": ports, "blackIps": black_ips}
        vulnerability = {"type": vul_type, "pocs_hosts": pocs_hosts}
        data = {"taskName": task_name, "asset": asset, "vulnerability": vulnerability, "options": options}
        return self._send_request("api/v1/startScan", data)

    def get_scan_progress(self, task_id):
        """
        获取扫描任务进度信息

        :param task_id: 扫描任务ID
        :type task_id: str
        :return: 返回扫描任务进度信息的JSON对象
        :rtype: dict
        """
        data = {
            "taskId": task_id
        }
        return self._send_request("api/v1/getProgress", data)

    def resume_scan(self, task_id, options={}):
        """
        恢复扫描任务

        :param task_id: 任务ID
        :type task_id: str
        :param options: 扫描选项，默认为{"queue": 0, "random": True, "rate": 100, "portscanmode": 0, "screenshot": False, "extracthost": False, "deepAnalysis": True}
        :type options: dict
        :return: 返回操作结果的JSON对象
        :rtype: dict
        """
        default_options = {
            "queue": 0,
            "random": True,
            "rate": 100,
            "portscanmode": 0,
            "screenshot": False,
            "extracthost": False,
            "deepAnalysis": True
        }
        #合并字典 两个字典合并为一个，其中 default_options 字典中的键值对优先级高于 options 字典中的键值对。
        #如果存在相同的键，则后面的字典中的值会覆盖前面的字典中的值。
        options = {**default_options, **options}

        data = {
            "taskId": task_id,
            "options": options
        }
        return self._send_request("api/v1/resumeScan", data)

    def stop_scan(self, task_id):
        """
        停止任务
        :param task_id: 任务ID
        :type task_id: str
        :return: 返回JSON对象，如下：
                 {
                     "success": true,
                     "message": "",
                     "data": {}
                 }
        :rtype: dict
        """
        data = {
            "taskId": task_id
        }
        return self._send_request("api/v1/stopScan", data)

    def get_tasks(self, page=1, size=10, order_by='created_time', order='desc'):
        """
        获取任务列表

        :param page: 页码，默认为 1
        :type page: int
        :param size: 每页数量，默认为 10
        :type size: int
        :param order_by: 排序字段，默认为 created_time
        :type order_by: str
        :param order: 排序方式，默认为 desc
        :type order: str
        :return: 返回任务列表的 JSON 对象
        :rtype: dict
        """
        data = {
            "options": {
                "page": {
                    "page": page,
                    "size": size
                },
                "order": {
                    order_by: order
                }
            }
        }

        return self._send_request("api/v1/getTasks", data)





